{ nix-config, config, lib, pkgs, inputs, ... }:
{
  nixpkgs = {
    config = {
      allowUnfree = true;
    };
    overlays = builtins.attrValues nix-config.overlays
               ++ [ inputs.emacs-overlay.overlays.default
                    inputs.audio.overlays.default ];
  };

  imports = with nix-config.homeModules; [
    mako
    mpv
    yazi
    hyprlock
    hypridle
	];

  home = {
    packages = lib.mkMerge [
      [
        inputs.affinity-nix.packages.x86_64-linux.photo
        inputs.affinity-nix.packages.x86_64-linux.publisher
        inputs.affinity-nix.packages.x86_64-linux.designer
        inputs.shikane.packages.x86_64-linux.default
        inputs.isd.packages.x86_64-linux.default
      ]

      (with nix-config.packages.x86_64-linux; [
        mioctl
        tal.bassline
        tal.j8
        tal.u-no-lx
        chair-audio.snare
        chair-audio.cymb
        the-usual-suspects.vavra
        the-usual-suspects.nodal-red
        the-usual-suspects.xenia
        the-usual-suspects.ostirus
        the-usual-suspects.osirus
        paulxstretch
      ])

      (with pkgs; [
        guix
        thonny

        bitwig-studio5-latest

        wayback
        just
        yt-dlp
        erdtree
        age
        figlet
        ffmpeg-full
        rlwrap
        pinentry
        gcr
        gnutar
        libnotify
        unzip
        zip
        stdenv
        tldr
        delta
        htop
        dconf
        dconf2nix
        readline
        gnupg
        imagemagick
        gnutls
        gettext
        xorg.libX11
        xorg.libxcb
        freetype
        gnumake
        flex
        bison
        mesa
        neofetch
        nvd
        nitch
        ripgrep-all
        silver-searcher
        maestral
        maestral-gui
        nix-index
        fstl
        godot_4
        sane-airscan
        xsane
        dbus
        gst_all_1.gstreamer
        alsa-oss
        powertop
        age
        sops

        nix-du

        usbutils
        pciutils
        util-linux
        coreutils
        binutils
        protonplus
        ispell
        (aspellWithDicts (dicts: with dicts; [ en en-computers en-science es fr ]))
        texlive.combined.scheme-full
        appimage-run
        libappimage
        transmission_4-gtk
        lsof
        guile
        python311
        python311Packages.pip
        python311Packages.mido
        python311Packages.simpleeval
        python311Packages.parsedatetime
        python311Packages.pytz
        python311Packages.pint
        python311Packages.babel
        python311Packages.requests
        python311Packages.distutils-extra
        python311Packages.distlib
        ruff

        (sbcl.withPackages (ps: with ps; [
          cffi
          arrow-macros
          alexandria
          serapeum
        ]))

        kdlfmt
        kdotool
        ccl
        racket
        gdu
        poppler_utils
        editorconfig-core-c
        cljfmt
        libxml2
        gdtoolkit_4
        openssl.out
        nixfmt-rfc-style
        wl-clipboard
        clipse
        maim
        php
        rustup
        shfmt
        html-tidy
        libsecret
        samba4Full
        shellcheck
        wayland-utils
        libffi.dev
        nodejs_18
        platformio-core
        icloudpd
        libheif.out
        openal
        pkg-config
        ftgl
        gperf
        numbat
        glpng
        nurl
        freecad
        procs
        rpi-imager
        caligula
        flatpak

        prusa-slicer

        # global node packages
        nodePackages.typescript-language-server
        nodePackages.vscode-langservers-extracted
        nodePackages.prettier
        nodePackages.javascript-typescript-langserver
        nodePackages.stylelint
        nodePackages.svelte-language-server
        nodePackages.typescript
        nodePackages.js-beautify

        nixd
        nixdoc
        nix-init

        git-annex

        fishPlugins.foreign-env
        fishPlugins.z
        fishPlugins.fzf-fish
        fishPlugins.bass
        fishPlugins.autopair
        fishPlugins.forgit
        fishPlugins.colored-man-pages
        imgcat
        via
        vial
        bitwarden-cli
        bitwarden-desktop
        tailscale
        trayscale
        font-manager
        scribus
        pdfarranger
        tdf
        pdfchain
        pdftk
        ghostscript
        zathura
        gpu-viewer
        mesa-demos

        gnomeExtensions.just-perfection
        gnomeExtensions.dash-to-dock
        gnomeExtensions.paperwm
        gnomeExtensions.custom-hot-corners-extended
        gnomeExtensions.media-controls
        gnomeExtensions.looking-glass-button
        gnomeExtensions.auto-power-profile
        gnomeExtensions.appindicator
        gnome-software
        gnome-tweaks
        gnome-photos
        gnome-keyring
        ulauncher
        wl-mirror
        nwg-look
        soundconverter
        # gtop
        wmctrl
        # colloid-gtk-theme
        colloid-icon-theme
        whitesur-gtk-theme
        whitesur-cursors
        whitesur-icon-theme
        bibata-cursors
        adwaita-qt

        icloudpd
        squirreldisk
        papers
        newsflash
        planify
        borgbackup
        vorta
        stirling-pdf
        trash-cli
        ripdrag
        wineWowPackages.stagingFull
        winetricks

        (yabridge.override {
          wine = wineWowPackages.stagingFull;
        })
        (yabridgectl.override {
          wine = wineWowPackages.stagingFull;
        })

        chayang
        sway-launcher-desktop
        swayosd
        spice
        spice-gtk
        spice-protocol
        virt-viewer
        virt-manager
        bfg-repo-cleaner
        thunderbird-latest
        emojify
        smile
        gnomeExtensions.smile-complementary-extension
        impression
        zoom-us
        nyxt
        ungoogled-chromium
        obsidian
        vlc
        polychromatic
        discord
        slack
        signal-desktop
        eyedropper
        calcurse
        onlyoffice-desktopeditors
        libreoffice
        hyphenDicts.en_US
        plex-mpv-shim
        plexamp
        rmpc
        resources
        carla
        switcheroo
        tenacity
        supercollider-with-plugins
        blender
        davinci-resolve-studio
        steam
        processing
        zotero
        jre
        tesseract
        handbrake
        renoise
        krita
        rawtherapee
        inkscape
        gimp
        epsonscan2
        kicad
        turbocase
        openscad-unstable
        openscad-lsp
        picard
        evolutionWithPlugins
        plugdata
        pavucontrol
        qpwgraph
        helvum
        coppwr
        alsa-utils
        alsa-tools
        clapper
        ardour
        dockfmt
        impala
        playerctl
        oculante
        door-knocker
        farge
        halloy
        paleta
        tree-sitter
        (tree-sitter.withPlugins (_: tree-sitter.allGrammars))

        # Audio plugins (LV2, VST2, VST3, LADSPA)
        eq10q
        geonkick
        bespokesynth
        helm
        surge
        odin2
        calf
        chow-kick
        chow-phaser
        chow-centaur
        chow-tape-model
        chow-multitool
        easyeffects
        vcv-rack
        cardinal
        roomeqwizard
        guitarix
        vital
        helio-workstation
        stochas
      ])
    ];

    username = "emenel";
    homeDirectory = "/home/emenel";

    sessionPath = [ "/home/emenel/.config/emacs/bin" "/home/emenel/.npm-packages/bin" "/home/emenel/Applications" "/home/emenel/.local/bin" "/home/emenel/.local/share" "/home/emenel/.nix-profile/lib"];

    sessionVariables = {
      WINEFSYNC = 1;
      PKG_CONFIG_PATH = "/home/emenel/.nix-profile/lib/pkgconfig:/home/emenel/.nix-profile/lib64/pkgconfig:/home/emenal/.nix-profile/share/pkgconfig";
      GI_TYPELIB_PATH = "/run/current-system/sw/lib/girepository-1.0";
      BW_CLIENTID = "$(cat ${config.sops.defaultSymlinkPath}/bw_client_id)";
      BW_CLIENTSECRET = "$(cat ${config.sops.defaultSymlinkPath}/bw_api_key)";
      NIXOS_OZONE_WL = "1";
      GSK_RENDERER = "ngl";
      MOZ_ENABLE_WAYLAND = 1;
    };

    pointerCursor = {
      package = pkgs.bibata-cursors;
      name = "Bibtata-Modern-Classic";
      size = 24;
      gtk.enable = true;
      x11.enable = true;
    };

    stateVersion = "24.05";
  };

  gtk.cursorTheme = {
    name = "Bibata-Modern-Classic";
    package = pkgs.bibata-cursors;
  };

  fonts = {
    fontconfig = {
      enable = true;
    };
  };

  sops = {
    age.keyFile = "/home/emenel/.config/sops/age/keys.txt"; # must have no password!

    defaultSopsFile = ./secrets.yaml;

    secrets = {
      bw_client_id = {
        path = "${config.sops.defaultSymlinkPath}/bw_client_id";
      };
      bw_api_key = {
        path = "${config.sops.defaultSymlinkPath}/bw_api_key";
      };
      ssh_key = {
        path = "${config.sops.defaultSymlinkPath}/ssh_key";
      };
      borg_url = {
        path = "${config.sops.defaultSymlinkPath}/borg_url";
      };
    };
  };

  accounts.email = {
    maildirBasePath = "/home/emenel/.maildir";
    accounts = {
      "matt@emenel.ca" = {
        address = "matt@emenel.ca";
        realName = "Matt Nish-Lapidus";
        primary = true;
        mu.enable = true;
        mbsync = {
          enable = true;
          create = "maildir";
        };
        msmtp.enable = true;
        flavor = "fastmail.com";
        passwordCommand = "rbw get mu4e-fastmail";
      };

      "admin@studyoftime.org" = {
        address = "admin@studyoftime.org";
        realName = "ISST Web Admin";
        mu.enable = true;
        mbsync = {
          enable = true;
          create = "maildir";
        };
        msmtp.enable = true;
        passwordCommand = "rbw get admin@studyoftime";
        imap.host = "imap.dreamhost.com";
        smtp.host = "imap.dreamhost.com";
        userName = "admin@studyoftime.org";
      };
    };
  };

  programs = {
    home-manager.enable = true;

    direnv = {
      enable = true;
      nix-direnv.enable = true;
    };
    fish = {
      enable = true;
      shellAliases = {
        "ls" = "eza";
        ".j" = "just -g";
        "em" = "emacsclient -n -r";
        "mkdir" = "mkdir -pv";
      };
      interactiveShellInit = ''
      bind --erase \ct
      '';
      plugins = [
        {
          name = "upto";
          src = pkgs.fetchFromGitHub {
            owner = "Markcial";
            repo = "upto";
            rev = "2d1f35453fb55747d50da8c1cb1809840f99a646";
            sha256 = "sha256-Lv2XtP2x9dkIkUUjMBWVpAs/l55Ztu7gIjKYH6ZzK4s=";
          };
        }
      ];
    };
    zoxide = {
      enable = true;
    };
    starship = {
      enable = true;
    };
    mu.enable = true;
    mbsync = {
      enable = true;
    };
    msmtp.enable = true;
    emacs = {
      enable = true;
      package = ((pkgs.emacsPackagesFor pkgs.emacs29-pgtk).emacsWithPackages (
        epkgs: [
          epkgs.vterm
          epkgs.all-the-icons
          epkgs.mu4e
          epkgs.sly
          epkgs.sly-quicklisp
          epkgs.geiser-racket
          epkgs.editorconfig
          epkgs.shfmt
          epkgs.nixpkgs-fmt
          epkgs.nixfmt
          epkgs.parinfer-rust-mode
          epkgs.manualPackages.tree-sitter-langs
          epkgs.manualPackages.treesit-grammars.with-all-grammars
          epkgs.pretty-sha-path
          epkgs.pdf-tools
        ]
      ));
    };
    ripgrep = {
      enable = true;
    };
    thefuck = {
      enable = true;
    };
    wezterm = {
      enable = true;
      package = inputs.wezterm.packages.${pkgs.system}.default;
      # extraConfig = lib.readFile ./dotfiles/dot_config/wezterm/wezterm.lua;
    };
    bat = {
      enable = true;
    };
    borgmatic = {
      enable = true;
      backups = {
        eddie = {
          location = {
            patterns = [
              "R /home/emenel"
              "- home/emenel/.cache"
              "- home/emenel/.nix*"
              "- home/emenel/.steam*"
              "- home/emenel/Dropbox"
              "- home/emenel/Sync"
              "- home/emenel/.dropbox*"
              "- home/emenel/.BitwigStudio"
              "- home/emenel/.local/share/bitwig"
            ];
            repositories = [
              {
                "path" = "ssh://oyi3ydnz@oyi3ydnz.repo.borgbase.com/./repo";
                "label" = "eddie on borgbase";
              }
            ];
          };
          retention = {
            keepDaily = 3;
            keepHourly = 3;
            keepMonthly = 2;
          };
        };
      };
    };
    eza = {
      enable = true;
    };
    fd = {
      enable = true;
    };
    fzf = {
      enable = true;
      enableFishIntegration = false;
    };
    broot = {
      enable = true;
    };
    git = {
      enable = true;
      lfs.enable = true;
    };
    htop.enable = true;
    pandoc.enable = true;
    password-store.enable = true;

    waybar = {
      enable = true;
      package = pkgs.waybar;
    };
    eww = {
      enable = true;
      configDir = ./dotfiles/dot_config/eww;
    };
    lazygit.enable = true;
    jq.enable = true;
    firefox.enable = true;
    rbw = {
      enable = true;
      package = nix-config.packages.x86_64-linux.rbw-latest;
    };
  };

  dconf.settings = {
    "org/gnome/desktop/background" = {
      picture-options = "none";
      primary-color = "#000000";
    };

    "org/gnome/desktop/interface" = {
      color-scheme = "prefer-dark";
    };

    "org/gnome/shell/keybindings" = {
      shift-overview-up = ["disabled"];
      shift-overview-down = ["disabled"];
      switch-to-application-1 = ["disabled"];
      switch-to-application-2 = ["disabled"];
      switch-to-application-3 = ["disabled"];
      switch-to-application-4 = ["disabled"];
      switch-to-application-5 = ["disabled"];
      switch-to-application-6 = ["disabled"];
      switch-to-application-7 = ["disabled"];
      switch-to-application-8 = ["disabled"];
      switch-to-application-9 = ["disabled"];
      switch-to-application-10 = ["disabled"];
    };

    "org/gnome/desktop/wm/keybindings" = {
      switch-to-workspace-up = ["disabled"];
      switch-to-workspace-down = ["disabled"];
      move-to-workspace-up = ["disabled"];
      move-to-workspace-down = ["disabled"];
    };
  };

  nixpkgs.config.packageOverrides = pkgs: {
    renoise = pkgs.renoise.override {
      releasePath = ../../assets/binary-installers/rns_344_linux_x86_64.tar.gz;
    };
  };

  services = {
    playerctld = {
      enable = true;
      package = pkgs.playerctl;
    };
    syncthing = {
      enable = true;
      # tray.enable = true;
    };
    borgmatic = {
      enable = true;
    };
    gpg-agent = {
      enable = true;
    };
    emacs = {
      enable = true;
      defaultEditor = true;
      package = ((pkgs.emacsPackagesFor pkgs.emacs29-pgtk).emacsWithPackages (
        epkgs: [
          epkgs.vterm
          epkgs.mu4e
          epkgs.sly
          epkgs.sly-quicklisp
          epkgs.geiser-racket
          epkgs.editorconfig
          epkgs.shfmt
          epkgs.nixpkgs-fmt
          epkgs.nixfmt
          epkgs.parinfer-rust-mode
          epkgs.manualPackages.tree-sitter-langs
          epkgs.manualPackages.treesit-grammars.with-all-grammars
          epkgs.pretty-sha-path
          epkgs.pdf-tools
        ]
      ));
      socketActivation.enable = true;
      client = {
        enable = true;
        arguments = ["-n" "-r"];
      };
    };
  };

  # stylix = {
  #   enable = true;
  #   base16Scheme = "${pkgs.base16-schemes}/share/themes/oxocarbon-dark.yaml";
  #   image = ./akira.png;
  #   cursor = {
  #     package = pkgs.bibata-cursors;
  #     name = "Bibtata-Modern-Classic";
  #     size = 24;
  #   };
  # };

  xdg.systemDirs.data = [
    "/home/emenel/.local/share/applications/wine/Programs"
    "/home/emenel/.local/share/applications"
    "/var/lib/flatpak/exports/share"
    "/home/emenel/.local/share/flatpak/exports/share"
    "/home/emenel/.nix-profile/share/applications"
  ];

  services.flatpak = {
    enable = true;
    packages = [
      "com.github.tchx84.Flatseal"
      "nl.hjdskes.gcolor3"
      "dev.qwery.AddWater"
      "tv.plex.PlexDesktop"
    ];
  };

  systemd.user = {
    enable = true;
    startServices = "sd-switch"; # auto reload services when home is rebuilt
    sessionVariables = {
      GSK_RENDERER = "ngl";
      MOZ_ENABLE_WAYLAND = 1;
    };
  };

  # moving files!
  xdg.configFile."halloy/config.toml".source = ./dotfiles/dot_config/halloy/config.toml;
  xdg.configFile."waybar".source = ./dotfiles/dot_config/waybar;
  xdg.configFile."starship.toml".source = ./dotfiles/dot_config/starship.toml;
  xdg.configFile."niri/config.kdl".source = ./dotfiles/dot_config/niri/config.kdl;

  xdg.configFile."wezterm".source = ./dotfiles/dot_config/wezterm;
  xdg.configFile."just/justfile".source = ./dotfiles/dot_config/just/justfile;
  xdg.configFile."git".source = ./dotfiles/dot_config/git;
  xdg.configFile."rbw".source = ./dotfiles/dot_config/rbw;
  xdg.configFile."shikane".source = ./dotfiles/dot_config/shikane;
  home.file.".ssh/config".source = ./dotfiles/dot_ssh/config;
  home.file.".npmrc".source = ./dotfiles/dot_npmrc;
  home.file.".vst3/yabridge/yabridge.toml".source = ./dotfiles/dot_vst3/yabridge.toml;
  home.file.".sbclrc".source = ./dotfiles/dot_sbclrc;
  home.file.".signature".source = ./dotfiles/dot_signature;
  home.file.".config/fish/functions" = {
    source = ./dotfiles/dot_config/fish/functions;
    recursive = true;
  };

  home.file.".config/fish/completions" = {
    source = ./dotfiles/dot_config/fish/completions;
    recursive = true;
  };

  xdg.configFile."emacs".enable = false;

  # home.file.".msmtprc".source = ./dotfiles/dot_msmtprc;
  # home.file.".mbsyncrc".source = ./dotfiles/dot_mbsyncrc;
  # xdg.configFile."isyncrc".enable = false;

  home.file.".local/bin" = {
    source = ./dotfiles/dot_local/bin;
    recursive = true;
  };

  home.file.".local/share/flf".source = ./dotfiles/dot_local/share/flf;

  home.file.".local/share/The Usual Suspects/Vavra" = {
    recursive = true;
    source = ../../packages/the-usual-suspects/vavra;
  };
  home.file.".local/share/The Usual Suspects/Xenia" = {
    recursive = true;
    source = ../../packages/the-usual-suspects/xenia;
  };
  home.file.".local/share/The Usual Suspects/Osirus" = {
    recursive = true;
    source = ../../packages/the-usual-suspects/osirus;
  };
  home.file.".local/share/The Usual Suspects/OsTIrus" = {
    recursive = true;
    source = ../../packages/the-usual-suspects/ostirus;
  };
  home.file.".local/share/The Usual Suspects/NodalRed2x" = {
    recursive = true;
    source = ../../packages/the-usual-suspects/nodal-red;
  };
}
